#include <iostream> 
#include <string> 
using namespace std;

template <typename T> 
class IVector {
public:
	virtual T at(int index) const = 0;
	virtual T front() const = 0;
	virtual T back() const = 0;
	virtual int size() const = 0;
	
	virtual void push_back(const T& value) = 0;
	virtual void pop_back() = 0;
	virtual void insert(int index, const T& value) = 0;
	virtual void erase(int index) = 0;
	virtual void clear() = 0;
};

template <typename T> 
class Vector : public IVector<T> {
	T* data;
	int _size;
	int _capacity;

	void expand()
	{
		_capacity <<= 1;
		T* expanded = new T[_capacity];

		for(int i=0; i < _size; ++i)
		{
			expanded[i] = data[i];
		}

		delete [] data;
		data = expanded;
	}
public:
	Vector()
	{
		_size = 0;
		_capacity = 4;
		data = new T[_capacity];
	}

	virtual ~Vector()
	{
		delete [] data;
	}

	Vector(const Vector& v)
	{
		_size = v._size;
		_capacity = v._capacity;
		data = new T[_capacity];

		for(int i=0; i < _size; ++i)
		{
			data[i] = v.data[i];
		}
	}

	Vector& operator=(const Vector& v)
	{
		if ( this != &v )
		{
			delete [] data;

			_size = v._size;
			_capacity = v._capacity;
			data = new T[_capacity];

			for(int i=0; i < _size; ++i)
			{
				data[i] = v.data[i];
			}
		}

		return *this;
	}

	virtual int size() const
	{
		return _size;
	}

	int capacity() const
	{
		return _capacity;
	}

	virtual T at(int index) const
	{
		return data[index];
	}

	virtual T front() const
	{
		return data[0];
	}

	virtual T back() const
	{
		return data[_size - 1];
	}

	virtual void push_back(const T& value)
	{
		while ( _size >= _capacity )
		{
			expand();
		}

		data[_size++] = value;
	}

	virtual void pop_back()
	{
		if ( _size > 0 )
		{
			--_size;
		}
	}

	virtual void insert(int index, const T& value)
	{
		while ( _size >= _capacity )
		{
			expand();
		}

		index = (index < 0) ? 0 : index;
		index = (index > _size) ? _size : index;

		for(int i = _size; i > index; --i)
		{
			data[i] = data[i-1];
		}

		++_size;

		data[index] = value;
	}

	virtual void erase(int index)
	{
		if ( index < 0 || index >= _size )
		{
			return;
		}

		for(int i = index; i < _size - 1; ++i)
		{
			data[i] = data[i+1];
		}

		--_size;
	}

	virtual void clear()
	{
		_size = 0;
	}
};